#include "readdata.h"
#include "varDoubleArr.h"
#include "globalVar.h"
#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include "numDifferentiation.h"
#include "dllFunctionDef.h"
#include "meanDataSets.h"
#include "writedata.h"

#include "calcCoefficients.h"
#include "dataSet.h"
#include "calcDeviation.h"

#include "solveODE.h"



//Read data from file
__declspec(dllexport) long __stdcall DLLReadData(char *cFilepath,long iDataSetNr, long lMode, long lConvKMH, long lNrOfReduct)
{	
	int iInit=0,iReadError;
	if (iInit=0) initalizeDataSets();
	iInit++;

	if (lMode==0)
	{	
		iReadError = readDataWithTimeStamp(cFilepath,&(arrDataSet[iDataSetNr].vData));
		if (lNrOfReduct>1) iReadError=reduceDataPoints(&(arrDataSet[iDataSetNr].vData),lNrOfReduct);
		if (lConvKMH==1) convertToMperS(&(arrDataSet[iDataSetNr].vData));
		return iReadError;
	}
	else
	{	
		iReadError = readData(cFilepath,&(arrDataSet[iDataSetNr].vData));
		if (lNrOfReduct>1) iReadError=reduceDataPoints(&(arrDataSet[iDataSetNr].vData),lNrOfReduct);
		if (lConvKMH==1) convertToMperS(&(arrDataSet[iDataSetNr].vData));
		return iReadError;
	}
}

__declspec(dllexport) long __stdcall DLLEqualDataSets(long lNrDataSets)
{
	return equalDataSets(0,lNrDataSets-1);
}

//Set parameters
__declspec(dllexport) void __stdcall DLLSetInputData( double dDynamicMass, double dSampleRate,long iDataSetNr) 
{	

	arrDataSet[iDataSetNr].dDynamicMass = dDynamicMass;
	arrDataSet[iDataSetNr].dSampleRate = dSampleRate;
}

 //Calculate the central difference
__declspec(dllexport) void __stdcall DLLCalcCentrDiff(long lDataSetNr)
{
	calcCentralDifference(&(arrDataSet[lDataSetNr].vData),&(arrDataSet[lDataSetNr].aData),arrDataSet[lDataSetNr].dSampleRate);
}

//Calculate coefficients
__declspec(dllexport) long __stdcall DLLCalcCouple(double *dResultArr, double dLambda, long lTotNrOfDataSets,long lCouplingMode)
{
	return calcCoupleCoefficients(dResultArr, 0, lTotNrOfDataSets-1, dLambda, lCouplingMode);
}

//Get the lenght,maximum and minimum of arrays
__declspec(dllexport) void __stdcall DLLPropOfArr(long *lLength, double *dMax, double *dMin, long lDataSetNr, long lArrCtr)  
{	
	switch(lArrCtr)
	{
		case 0: 
			*(lLength)= arrDataSet[lDataSetNr].vData.liArrLength;
			*dMax = findMax(&(arrDataSet[lDataSetNr].vData));
			*dMin = findMin(&(arrDataSet[lDataSetNr].vData));
			break;

		case 1:
			*(lLength)= arrDataSet[lDataSetNr].aData.liArrLength;
			*dMax = findMax(&(arrDataSet[lDataSetNr].aData));
			*dMin = findMin(&(arrDataSet[lDataSetNr].aData));
			break;

		case 2:
			*(lLength)= arrDataSet[lDataSetNr].resData.liArrLength;
			*dMax = findMax(&(arrDataSet[lDataSetNr].resData));
			*dMin = findMin(&(arrDataSet[lDataSetNr].resData));
			break;
	
	}

}

//Memory copy from C-Array to VB-Array
__declspec(dllexport) void __stdcall DLLMemoryCopy(double *dArrBeginning, long lDataSetNr, long lCount, long lArrCtr)
{	
	switch(lArrCtr)
	{
		case 0: 
			memcpy(dArrBeginning,(arrDataSet[lDataSetNr].vData.pfArr),lCount*sizeof(double));
			break;
		case 1:
			memcpy(dArrBeginning,(arrDataSet[lDataSetNr].aData.pfArr),lCount*sizeof(double));
			break;
		case 2:
			memcpy(dArrBeginning,(arrDataSet[lDataSetNr].resData.pfArr),lCount*sizeof(double));
			break;
	}
}

//Solve ODE with Runge-Kutta
__declspec(dllexport) void __stdcall DLLSolveODE(double dA0,double dA2,double dM,double dStep,long lDataSetsNr)  
{
		solveODE(&(arrDataSet[lDataSetsNr]),dA0,dA2,dM,dStep);	
}

//Calculate error
__declspec(dllexport) double __stdcall DLLSquareDev(long lDataSetsNr)  
{
	return calcSquareDeviation(&(arrDataSet[lDataSetsNr]));
}

//Mean datasets
__declspec(dllexport) long __stdcall DLLMeanDataSets (char *cFilepath,long lNrDataSets)
{	
	int iError=1;

	if (meanDataSets(0,lNrDataSets-1,&(arrDataSet[lNrDataSets].vData))==0)
	{
		if (writeData(cFilepath,&(arrDataSet[lNrDataSets]).vData)==0) iError=0;	
	}
	
	return iError;
}


//Reset the data sets
__declspec(dllexport) void __stdcall DLLDelDataSet()  
{
	resetAllDataSets();
}



