#include "nr_d.h"
#include "nrutil_d.h"

/** this function inverts the matrix given by ppfMatrixToInv
 *  arguments: ppfMatrixToInv - pointer to matrix which will be inverted
 *             the inverted matrix will be stored in ppfMatrixToInv
 *             iDimension - dimension of the matrix
 **/
int invertMatrix(double** ppfMatrixToInv, int iDimension)
{
  int m=1;
  int iErrorHand;      
  double **b;       //dummy right-hand side for gaussj(...)
  b = dmatrix(1,iDimension,1,1);
  iErrorHand = gaussj2(ppfMatrixToInv,iDimension,b,m);
  free_dmatrix(b,1,iDimension,1,1);
  return iErrorHand;
}

/** this function multiplies the given matrices and stores the result in
 *   ppfResult, dimensions of the result matrix stored in iRows, iCol
 *  arguments: ppfMatrixLeft - lefthandside matrix
               ppfMatrixRight - righthandside matrix 
    return value: int: 0 - successfull, 1 - error
 **/
int multMatrix(double** ppfMatrixLeft, int iRowsLeft, int iColLeft,
               double** ppfMatrixRight, int iRowsRight, int iColRight,
               double*** ppfResult, int *iRows, int *iCol)
{
  if (iColLeft == iRowsRight)
  {
    int i,j,k,n;
    n = iColLeft;
    *iRows = iRowsLeft;             //result Rows
    *iCol = iColRight;              //result Columns
    *ppfResult = dmatrix(1,*iRows,1,*iCol);

    for (i=1;i<=iRowsLeft;i++)
    {
      for (k=1;k<=iColRight;k++)
      {
        (*ppfResult)[i][k] = 0.0; //initialize element

        for(j=1;j<=n;j++)
        {
          (*ppfResult)[i][k] += ppfMatrixLeft[i][j] * ppfMatrixRight[j][k];
        }
      }
    }

    return 0;
  }
  else
  {
    return 1;
  }   
}