#include <stdio.h>
#include <math.h>
#include "solveODE.h"
#include "varDoubleArr.h"

/**This function solves the Ordinary Differential Equations 
	first order numerically**/


void solveODE (dataSet* pToDataSet,double dA0,double dA2,double dM,double dStep)
{
	long i;	//coefficents for ODE
	double dV0,dK1,dK2,dK3,dK4;
	//varDoubleArr dT={0,0,NULL},dVAnaly={0,0,NULL};

	dV0=pToDataSet->vData.pfArr[0]; //Set start velocity

	//initialize dataset for the solution of ODE
	freeArray(&(pToDataSet->resData));
	pToDataSet->resData.iMaxLength = 0;
    pToDataSet->resData.liArrLength = 0;
    pToDataSet->resData.pfArr = NULL;

	addItem(&(pToDataSet->resData),pToDataSet->vData.pfArr[0]);  //Set start velocity
	
	for (i=1;i<(pToDataSet->vData.liArrLength);i++)
	{	
		//calc coefficients for Runge-Kutta
		dK1=functionODE(dA0,dA2,dStep,pToDataSet->resData.pfArr[i-1],dM,0);			
		dK2=functionODE(dA0,dA2,dStep,pToDataSet->resData.pfArr[i-1],dM,dK1*0.5);
		dK3=functionODE(dA0,dA2,dStep,pToDataSet->resData.pfArr[i-1],dM,dK2*0.5);
		dK4=functionODE(dA0,dA2,dStep,pToDataSet->resData.pfArr[i-1],dM,dK3);
		
		//add velocity solved with Runge-Kutta
		addItem(&(pToDataSet->resData),(pToDataSet->resData.pfArr[i-1]+((dK1+(2*dK2)+(2*dK3)+dK4))/6.0));

		//dT[i]=dT[i-1]+dStep;								//increment time
		//dV[i]=dV[i-1]+((dK1+(2*dK2)+(2*dK3)+dK4))/6.0;	//calc v(t) with Runge-Kutta method
		//dVAnaly[i]=sqrt(dA0/dA2)*tan(-sqrt(dA0*dA2)*(dT[i]/dM)+atan(dV0*sqrt(dA2/dA0)));
	}

}

double functionODE(double dA0, double dA2, double dStep, double dV,double dM, double dK)
{
	return dStep*(dA0+dA2*pow(dV+dK,2))/(-dM); //returns coefficients for Runge-Kutta
}