#include "varDoubleArr.h"
#include <stdlib.h>

/** this function adds an item to the variable double array (structure varDoubleArr)
 *  arguments: pointer to the varDoubleArr; value which has to be added
 *  return value: int -> 0: successfull; 1:error - memory could not be allocated
 **/

int addItem (varDoubleArr *pToStruct, double fValue)
{ 
  //if no more capacity is available - new reallocation of memory
  if(pToStruct->iMaxLength <= pToStruct->liArrLength)
  {
    //reallocation successfull?
    if(NULL == (pToStruct->pfArr = (double*)realloc(pToStruct->pfArr,
                                           sizeof(double) *
                                            (pToStruct->iMaxLength + VDA_REALLOC_AMOUNT))))
    {
      return 1; //error
    }
    //memory in number of possible doubles increased by the number of reallocated doubles
    pToStruct->iMaxLength += VDA_REALLOC_AMOUNT;
  }
  pToStruct->pfArr[pToStruct->liArrLength]=fValue; //add the value into the array
  pToStruct->liArrLength +=1; //contained elements increased
  return 0; //successfully done
}


/** this function removes an element form a double array (structure varDoubleArr)
 *  arguments: pointer to the varDoubleArr, index of the element which should be 
 *  removed
 *  return value: int -> 0: successfull; 1:error - index does not exist
 **/

int removeItem(varDoubleArr *pToStruct, long liIndex)
{ 
  long i = 0;
  
  if (liIndex <pToStruct->liArrLength && liIndex >= 0)
  {
    for(i = liIndex+1; i<pToStruct->liArrLength; i++)
    {
      //move all elements with higher index as liIndex to the left
      pToStruct->pfArr[i-1] = pToStruct->pfArr[i]; 
    }
    pToStruct->liArrLength -= 1; //reduce array length
    return 0;
  }
  else
  {
    return 1;  //index does not exist
  }
  
}

/** this function finds the maximum in a double array (structure varDoubleArr)
 *  arguments: pointer to the varDoubleArr
 *  return value: maximum (double)
 **/

double findMax(varDoubleArr *pToStruct)
{
  long i = 0;
  double fMax; //maximum
  fMax = *(pToStruct->pfArr); //initialize max with first element of the array

  for (i=0;i<pToStruct->liArrLength;i++) //loop: all elements
  {
    if(*(pToStruct->pfArr+i) > fMax) //if next element is larger than max 
    {
      fMax = *(pToStruct->pfArr+i);  //-> new max
    }
  }
  return fMax; //returns the maximum as double
}

/** this function finds the minimum in a double array (structure varDoubleArr)
 *  arguments: pointer to the varDoubleArr
 *  return value: minimum (double)
 **/
double findMin(varDoubleArr *pToStruct)
{
  long i = 0;
  double fMin; //minimum
  fMin = *(pToStruct->pfArr); //initialize min with first element of the array

  for (i=0;i<pToStruct->liArrLength;i++) //loop: all elements
  {
    if(*(pToStruct->pfArr+i) < fMin) //if next element is smaller than max 
    {
      fMin = *(pToStruct->pfArr+i); //-> new min
    }
  }
  return fMin; //returns the minimum as double
}

/** this function disallocates the memory block of a 
 *  variable double array (structure varDoubleArr) and sets the length 0
 *  arguments: pointer to the varDoubleArr
 *  return value: non
 **/

void freeArray(varDoubleArr *pToStruct)
{  
  free(pToStruct->pfArr);
  pToStruct -> pfArr = NULL;
  pToStruct->liArrLength = 0;
  pToStruct->iMaxLength = 0;
}

void convertToMperS(varDoubleArr *pToStruct)
{
  int i;

  for (i=0;i<pToStruct->liArrLength;i++)
  {
    pToStruct->pfArr[i] /= 3.6;
  }
}

int reduceDataPoints(varDoubleArr *pToStruct, long lNrOfReduct)
{
  int i,j,iOldLength;
  double dSum=0;

	if (5*lNrOfReduct<pToStruct->liArrLength)
	{
	  for (i=0;i<pToStruct->liArrLength;i+=lNrOfReduct)	//average datapoints
	  {	
		  for (j=i;j<i+lNrOfReduct;j++)
		  {
			  dSum+=pToStruct->pfArr[j];
		  }

		pToStruct->pfArr[i/lNrOfReduct] = dSum/lNrOfReduct;
		dSum=0;
	  }

	  iOldLength=pToStruct->liArrLength;//remove further data points

	  for (i=pToStruct->liArrLength-1;i>iOldLength/lNrOfReduct-1;i--)
	  {
		removeItem(pToStruct,i);
	  }
	  return 0; //no error
	}
	else return 2; //error

}

