Attribute VB_Name = "mdCalculate"
Option Explicit

Private Declare Sub SetInputData Lib _
"V33.dll" _
Alias "DLLSetInputData" (ByVal dMass As Double, ByVal dSamplingRate As Double, ByVal lDataSetNR As Long)

Private Declare Sub CalcCentrDiff Lib _
"V33.dll" _
Alias "DLLCalcCentrDiff" (ByVal lDataSetNR As Long)

Private Declare Function CalcCouple Lib _
"V33.dll" _
Alias "DLLCalcCouple" (ByRef dResult As Double, ByVal dLambda As Double, ByVal lDataSetNR As Long, ByVal lCouplingMode As Long) As Long

Public Declare Sub DelDataSets Lib _
"V33.dll" _
Alias "DLLDelDataSet" ()

Public Declare Sub SolveODE Lib _
"V33.dll" _
Alias "DLLSolveODE" (ByVal dA0 As Double, ByVal dA2 As Double, ByVal dM As Double, ByVal dSampleRate As Double, ByVal lDataSetsNr As Long)

Private Declare Function EqualDataSets Lib _
"V33.dll" _
Alias "DLLEqualDataSets" (ByVal iNrDataSet As Long) As Long

Private Declare Function SquareDev Lib _
"V33.dll" _
Alias "DLLSquareDev" (ByVal iDataSetNr As Long) As Double

Sub RunCalculation()
    
    Dim iCalcResult As Integer
    Dim NumberOfSelectedDS As Long
    Dim lReadError As Long
    Dim l As Long
    Dim blnReadSuccessful As Boolean
    Dim iEqualDataSets As Integer
    
    DelDataSets                                     'Delete all datasets
    ResetGraphs                                     'Clear graph
    NumberOfSelectedDS = SelectionCheck             'Get the number of selected files
    bDraw = False
    
    If NumberOfSelectedDS > 0 Then                  'Minimum one selected file
        If (NumberOfSelectedDS < 2 And iCouple > 0) Then    'Coupled and too less datasets
            MsgBox ("For coupling you need at least two datasets!"), vbInformation
        Else
            If PreCalculateCheck(NumberOfSelectedDS) = 0 Then   'PreCalculate function worked without error
                
                For l = 0 To NumberOfSelectedDS - 1
                    lReadError = ReadDataSets(l, typDataset(iSelectedFiles(l)).FileFormat)
                    If lReadError = 0 Then        'Read the data sets in sequence
                            blnReadSuccessful = True
                            If typDataset(iSelectedFiles(l)).Status = 0 Then typDataset(iSelectedFiles(l)).Status = 1  'Readdata was successful
                    ElseIf lReadError = 2 Then          'Data reduction error
                            blnReadSuccessful = False
                            MsgBox ("Smooth of dataset " & typDataset(iSelectedFiles(l)).FileName & " was not successful, factor of data reduction is too high!"), vbCritical
                            DelDataSets                 'Reset all data sets in dll
                            Exit Sub
                    Else:   blnReadSuccessful = False
                            MsgBox ("Read of dataset " & typDataset(iSelectedFiles(l)).FileName & " was not successful!"), vbCritical
                            DelDataSets                 'Reset all data sets in dll
                            Exit Sub
                    End If
                Next
                
                If (NumberOfSelectedDS > 1 And iCouple > 0) Then    'Equalize data sets if coupled and number>2
                    If PlausabilityCheck(NumberOfSelectedDS) < 1 Then
                        If (EqualDataSets(NumberOfSelectedDS)) > 1 Then   'Error Variation of data sets to big
                            blnReadSuccessful = False
                            MsgBox ("Calculation-Error, variation of data sets is too big!"), vbCritical
                        End If
                    Else
                        blnReadSuccessful = False
                        MsgBox ("Files have not the same sample-rate!"), vbCritical
                        DelDataSets                 'Reset all data sets in dll
                        Exit Sub
                    End If
                        
                End If
                    
                
                If blnReadSuccessful = True Then        'All datasets added succesfully
                    
                   If Calculate(NumberOfSelectedDS) = 0 Then  'Selected data sets calculated with no error
                        
                        For l = 0 To NumberOfSelectedDS - 1
                            MemCopy1 l                       'Copy memory from C-Array to VB-Array
                            MemCopy2 l
                            MemCopy3 l
                        Next
                                                
                        Display NumberOfSelectedDS          'Display in flex-grid
                        DelDataSets                         'Reset all data sets in dll
                        RunDraw                             'Graph
                   
                    Else:
                    MsgBox ("Calculation-Error, singular matrix! Check file and input parameters"), vbCritical
                    End If
                End If
            End If
        End If
    Else: MsgBox ("No files selected!"), vbInformation
    End If
    
End Sub

Private Function PreCalculateCheck(ByVal NumberOfSelectedDS As Long) As Integer

Dim iNrNoInputData As Integer
iNrNoInputData = -1
        
    If (InputDataCheck(NumberOfSelectedDS, iNrNoInputData)) = 0 Then   'Input data added to dataset?
        'Plausability
        PreCalculateCheck = 0                           'ReadDataSets NumberOfSelectedDS
        bCalcAutomaticAfterDatainput = 0
    Else:
        MsgBox ("No input-data for the selected files!"), vbInformation
        frmMain.Enabled = False
        frmDataInput.cmbDataInput.ListIndex = iNrNoInputData
        DoEvents
        frmDataInput.Show vbModal                               'No input data for selected files
        PreCalculateCheck = 1
    End If
    
End Function

Private Function InputDataCheck(ByVal lNumber As Long, ByRef iNrNoInputData As Integer) As Integer
    
    Dim l As Long
    Dim iNoInputData As Integer
    
    For l = 0 To lNumber - 1
        If typDataset(iSelectedFiles(l)).Status < 2 Then
            iNoInputData = 1                               'No input data for selected files
            If iNrNoInputData = -1 Then iNrNoInputData = iSelectedFiles(l)
        End If
    Next
    
    InputDataCheck = iNoInputData
    
End Function

Private Function Calculate(lNumber As Long) As Long
    
    Dim l, lCalcResult As Long
    Dim i As Integer
    Dim dResultArr() As Double
    
    For l = 0 To lNumber - 1
        With typDataset(iSelectedFiles(l))
            'Set data to the structure in dll
            If .DynamicMass = 0 Then .DynamicMass = .Mass * 1.03
            Call SetInputData(.DynamicMass, .SampleRate, l)
            Call CalcCentrDiff(l)  'Calculate CentralDifference
            
            If iCouple = 2 Then
            
                iNrOfLambdas = frmOptions.sldLambdaVar.Value
                
                ReDim .A0.darrData(iNrOfLambdas - 1)      'Redimensionize arrays for coefficients
                ReDim .A2.darrData(iNrOfLambdas - 1)      'Lambda is variable
                ReDim .Q.darrData(iNrOfLambdas - 1)
                ReDim .Cd.darrData(iNrOfLambdas - 1)
                ReDim .fr0.darrData(iNrOfLambdas - 1)
                
            Else
                ReDim .A0.darrData(0)       'Redimensionize arrays for coefficients
                ReDim .A2.darrData(0)       'Lambda is null or constant
                ReDim .Q.darrData(0)
                ReDim .Cd.darrData(0)
                ReDim .fr0.darrData(0)
                
                iNrOfLambdas = 1
            End If
        End With
    Next
    
    If iCouple = 0 Then
        ReDim dResultArr(2 * NUMOFMAXFILES, 1)
        'Calculate without coupling, lambda=0
        lCalcResult = CalcCouple(dResultArr(0, 0), 0, lNumber, 0)
        If lCalcResult > 0 Then     'error at couple calculate
            Calculate = lCalcResult
            Exit Function
        End If
    ElseIf iCouple = 1 Then
        ReDim dResultArr(2 * NUMOFMAXFILES, 1)
        'Calculate with coupling, lambda=const
        lCalcResult = CalcCouple(dResultArr(0, 0), dLambda(0), lNumber, lCouplingMode)
        If lCalcResult > 0 Then     'error at couple calculate
            Calculate = lCalcResult
            Exit Function
        End If
    ElseIf iCouple = 2 Then
        ReDim dResultArr(2 * NUMOFMAXFILES, iNrOfLambdas - 1)
        For l = 0 To iNrOfLambdas - 1
             'Calculate with coupling, lambda=variable
            lCalcResult = CalcCouple(dResultArr(0, l), dLambda(l), lNumber, lCouplingMode)
            If lCalcResult > 0 Then 'error at couple calculate
                Calculate = lCalcResult
                Exit Function
            End If
        Next
    End If
    
    'Get coefficient out of the result-array
    
    For i = 0 To iNrOfLambdas - 1
        
        For l = 0 To (lNumber - 1) * 2 Step 2
            typDataset(iSelectedFiles(l / 2)).A0.darrData(i) = dResultArr(l, i)
        Next
    
        For l = 1 To (lNumber) * 2 Step 2
            typDataset(iSelectedFiles((l - 1) / 2)).A2.darrData(i) = dResultArr(l, i)
        Next
    
        For l = 0 To lNumber - 1
            With typDataset(iSelectedFiles(l))
                SolveODE .A0.darrData(i), .A2.darrData(i), .DynamicMass, .SampleRate, l
                .Q.darrData(i) = SquareDev(l)     'Calculate the deviation of ODE
            End With
        Next

        'Calculate output data
        For l = 0 To lNumber - 1
            With typDataset(iSelectedFiles(l))
            
                .Density = .Pressure / (.Temperature * 287.05)
                .Cd.darrData(i) = 2 * .A2.darrData(i) / (.Density * .FrontalArea)
                .fr0.darrData(i) = .A0.darrData(i) / (.Mass * 9.81)
                
                'Set status
                Select Case iCouple
                    Case 0: .Status = 3     'lambda = 0
                    Case 1: .Status = 4     'lambda = const
                    Case 2: .Status = 5     'lambda = variable
                End Select
                
            End With
        Next
   
    Next
    
End Function


